#include "framecpp/config.h"

#include <algorithm>

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"

#include "framecpp/Version3/FrStatData.hh"
#include "framecpp/Version3/FrDetector.hh"
#include "framecpp/Version3/FrSE.hh"
#include "framecpp/Version3/FrSH.hh"
#include "framecpp/Version3/PTR_STRUCT.hh"

using namespace FrameCPP::Version_3;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

//=======================================================================
// FrStatData
//=======================================================================

const INT_4U FrStatData::ALL_VERSIONS   = 0xFFFFFFFF;
const INT_4U FrStatData::LATEST_VERSION = 0xFFFFFFFE;

FrStatData::
FrStatData( )
  : Common::FrStatData( StructDescription( ) )
{
}

FrStatData::
FrStatData( const FrStatData& Source )
  : Common::FrStatData( StructDescription( ) ),
    FrStatDataStorage( Source ),
    FrStatDataRefs( Source )
{
}

FrStatData::
FrStatData( const std::string& name,
	    const std::string& comment,
	    const INT_4U timeStart,
	    const INT_4U timeEnd,
	    const INT_4U version )
  : Common::FrStatData( StructDescription( ) ),
    FrStatDataStorage( name, comment,
		       timeStart, timeEnd, version )
{
}

FrStatData::
FrStatData( IStream& Stream )
  : Common::FrStatData( StructDescription( ) )
{
  m_data( Stream );
  m_refs( Stream );
}

FrameCPP::cmn_streamsize_type FrStatData::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.Bytes( )
    + m_refs.Bytes( Stream )
    ;
}

FrStatData* FrStatData::
Clone( ) const
{
  return new FrStatData( *this );
}

FrStatData* FrStatData::
Create( IStream& Stream ) const
{
  return new FrStatData( Stream );
}

const char* const FrStatData::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrStatData::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {
    ret( FrSH( FrStatData::StructName( ), FrStatData::STRUCT_ID,
	       "Static Data Structure" ) );

    FrStatDataStorage::data_type::Describe( ret );
    ref_type::Describe( ret );
  }

  return &ret;
}

void FrStatData::
Write( OStream& Stream ) const
{
  m_data( Stream );
  m_refs( Stream );
}

FrStatData& FrStatData::
Merge( const FrStatData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}


bool FrStatData::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

std::auto_ptr< FrameSpec::Object > FrStatData::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrStatData* >( this ) );
  }
  throw
    Unimplemented( "Object* FrStatData::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrStatData::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}
